function [ modelFunc_Y_convolved_bins ] = calcModelFuncConvolved( convFunc, convRad_N, modelFuncType, modelFunc_sampling_X, modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, modelFunc_dX, dX_refinement, histBinN, densFuncSize, densFuncMax, densFuncBG, pos, varargin )
% This function calculates the model function to be fit on the histogram
% data of the measurement. First, calls a function that calculates the
% function values of the supposed binding site density, then convolves it
% with the inaccuracies arising from the localization precision and the
% linker length, finally numerically integrates it over the histogram bins.
% 
% parents: lineDiameter_fit
% children: calcModelFunc
% 
% INPUT VARIABLES
% convFunc: sampled values of the function with that the binding site density should be convolved
% convRad_N: half the points of the convolution function minus 0.5
% modelFuncType: model function of the supposed binding site density
% modelFunc_sampling_X: sampling point coordinates of the model function
% modelFunc_sampBoundaries_left: left boundary coordinates of the sampling bins
% modelFunc_sampBoundaries_right: right boundary coordinates of the sampling bins
% modelFunc_dX: size of the sampling bins
% dX_refinement: number of the sampling points within each bin, sampling refinement for the numerical integration
% histBinN: number of bins in the histogram
% densFuncSize: the FWHM (full width at half-maximum) of the supposed binding site density
% densFuncMax: value of the binding site density function at its central position (maximum for every function except the "circle" model function)
% densFuncBG: constant background of the binding site density function
% pos: central position of the binding site density function

%% Model function prior the convolution

[ modelFunc_Y ] = calcModelFunc( convRad_N, modelFuncType, modelFunc_sampling_X, modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, densFuncSize, densFuncMax, densFuncBG, pos, varargin{:} );

%% Convolution

% for testing, covlolution with a top-hat function:
%convFuncSize=2*convRad_N+1;
%convFunc=ones(convFuncSize,1);

% initialization of the convolved model function
modelFunc_Y_convolved=zeros(size(modelFunc_sampBoundaries_left));
% numerical calculation of the convolution
for idxSamp=(convRad_N+1):(size(modelFunc_Y_convolved,1)+convRad_N)
    modelFunc_Y_convolved(idxSamp-convRad_N)=sum(modelFunc_Y(idxSamp-convRad_N:idxSamp+convRad_N).*convFunc)*modelFunc_dX;
end

% for testing, comparison of the model function prior and after the
% convolution:
%figure()
%plot(modelFunc_sampling_X, modelFunc_Y)

%% Numerical integration of the convolved model function, for comparison with the measured histogram data

% indices of the model function values belonging to the first bin
intIndices_firstBin=1:dX_refinement;

% for each bin, these values needs be added to the aforementioned indices
% to get the indices of the model function values belonging to the bins
intIndices_binIncrement=((1:histBinN)'-1)*dX_refinement;

% indices of the model function values belonging to the bins
linIndices_forPixelization=repmat(intIndices_firstBin,histBinN,1)+repmat(intIndices_binIncrement,1,dX_refinement);

% constants for the numerical integration
numIntConstant=ones(size(linIndices_forPixelization))/dX_refinement;

% model function values integrated over the bins
modelFunc_Y_convolved_bins=sum(numIntConstant.*modelFunc_Y_convolved(linIndices_forPixelization),2);


end

