function [ fittedParamsTable, fittedHistFigure ] = lineDiameter_fit( lineDiam_histData, curveFitTool, ...
    linkerType, sampleType, varargin )
% This function reads the histogram data of the measured distances and
% fits a model function to it. Can be used for determining a line diameter.
% At first, a dialogue windows is asking the fitting parameters.
% Then function prepares the 'central curve-localization' data. If
% no localization precision filtering is applied and the histogram edges
% match for all lines, then the histogram data is used, otherwise, new
% histogram is created from the raw "central curve-localization" data. The
% sampling points for the convolution and for the model function are also
% calculated.
% In the next step the model function fitting is performed. During the fit,
% the iteration changes the binding site distribution function, then its is
% convolved with the localization inaccuracies and compared with the raw
% data of the 'central curve-localization' distribution. In case of a 
% single line sample the initial width is derived from the standard
% deviation of the histogram data while in case of a double line sample, it
% is determined by a double Guassian fit. In case a single line sample the
% model function widths are also calculated from the standard deviation.
% First the standard deviation of the fitted constant background is
% subtracted from the standard deviation of the data, then the the width
% is calculated for the 'Gaussian', 'Rectangular' and 'Disk' model
% functions. Finally a figure is created showing the histogram data, the
% fitted model function and the fitted binding site density function. The
% fitted model function widths ("FWHM"), the residual of the fit and in case
% of double line sample, the distance of the lines are also written out on
% the figure.

% parents: rainSTORM_extras_lineDiameter_fit
% children: lineDiameter_fit_inputDialog, calcConvFunction, calcModelFuncConvolved

% INPUT VARIABLES
% lineDiam_histData: raw data of the localization-central line distances, its localization precision and of the histogram created from it
% linkerType: type of distribution associated with the linker length, can be 'sphere' or 'Gaussian'
% twoLinesBool: boolean for whether the fit should be performed on a double line
% varargin: binding site density parameters 

    function res=fitDist(histCounts, convFunc, convRad_N, modelFuncType, modelFunc_sampling_X, modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, modelFunc_dX, dX_refinement, histBinN, x )
        % function to fit the model function to histogram data of the measured distances
        
        % parameters to fit
        densFuncSize=x(1);
        densFuncMax=x(2);
        densFuncBG=x(3);
        
        % values of the model function
        sample_Y_convolved=calcModelFuncConvolved( convFunc, convRad_N, modelFuncType, modelFunc_sampling_X, modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, modelFunc_dX, dX_refinement, histBinN, densFuncSize, densFuncMax, densFuncBG, 0 );
        % residual of the fitted model function
        res=sum((sample_Y_convolved-histCounts).^2)/sum(histCounts.^2);      % mean squared error as the residue
        %res=sum((sample_Y_convolved-histCounts).^2)/length(modelFunc_sampling_X);
    end

    function res=fitDist_twoLines(histCounts, convFunc, convRad_N, modelFuncType, modelFunc_sampling_X, modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, modelFunc_dX, dX_refinement, histBinN, x )
        % function to fit the model function to histogram data of the measured distances
        
        % parameters to fit
        densFuncSize=x(1);
        densFuncMax=x(2);
        densFuncBG=x(3);
        pos_1=x(4);
        pos_2=x(5);
        
        % values of the model function
        sample_Y_convolved_1=calcModelFuncConvolved( convFunc, convRad_N, modelFuncType, modelFunc_sampling_X, modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, modelFunc_dX, dX_refinement, histBinN, densFuncSize, densFuncMax, densFuncBG, pos_1 );
        sample_Y_convolved_2=calcModelFuncConvolved( convFunc, convRad_N, modelFuncType, modelFunc_sampling_X, modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, modelFunc_dX, dX_refinement, histBinN, densFuncSize, densFuncMax, 0, pos_2 );
        sample_Y_convolved=sample_Y_convolved_1+sample_Y_convolved_2;
        % residual of the fitted model function
        res=sum((sample_Y_convolved-histCounts).^2)/sum(histCounts.^2);      % mean squared error as the residue
        %res=sum((sample_Y_convolved-histCounts).^2)/length(modelFunc_sampling_X);
    end

    function res=fitDist_twoLines_2(histCounts, convFunc, convRad_N, modelFuncType, modelFunc_sampling_X, modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, modelFunc_dX, dX_refinement, histBinN, x )
        % function to fit the model function to histogram data of the measured distances
        
        % parameters to fit
        densFuncSize=x(1);
        densFuncMax_1=x(2);
        densFuncMax_2=x(3);
        densFuncBG=x(4);
        pos_1=x(5);
        pos_2=x(6);
        
        % values of the model function
        sample_Y_convolved_1=calcModelFuncConvolved( convFunc, convRad_N, modelFuncType, modelFunc_sampling_X, modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, modelFunc_dX, dX_refinement, histBinN, densFuncSize, densFuncMax_1, densFuncBG, pos_1 );
        sample_Y_convolved_2=calcModelFuncConvolved( convFunc, convRad_N, modelFuncType, modelFunc_sampling_X, modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, modelFunc_dX, dX_refinement, histBinN, densFuncSize, densFuncMax_2, 0, pos_2 );
        sample_Y_convolved=sample_Y_convolved_1+sample_Y_convolved_2;
        % residual of the fitted model function
        res=sum((sample_Y_convolved-histCounts).^2)/sum(histCounts.^2);      % mean squared error as the residue
        %res=sum((sample_Y_convolved-histCounts).^2)/length(modelFunc_sampling_X);
    end

    function res=fitDist_gap(histCounts, convFunc, convRad_N, modelFunc_sampling_X, modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, modelFunc_dX, dX_refinement, histBinN, rimWidth, x )
        % function to fit the model function to histogram data of the measured distances
        
        % parameters to fit
        densFuncSize=x(1);
        densFuncMax=x(2);
        densFuncBG=x(3);
        if rimWidth==0
            rimHeight=0;
        else
            rimHeight=x(4);
        end
        
%         densFuncSize=110;
%         densFuncMax=-60;
%         densFuncBG=92;
%         rimHeight=x(4);
        
        rimDist=(densFuncSize+rimWidth)/2;
        
        % values of the model function
        sample_Y_convolved=calcModelFuncConvolved( convFunc, convRad_N, 'Rectangular gap', modelFunc_sampling_X, modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, modelFunc_dX, dX_refinement, histBinN, densFuncSize, densFuncMax, 0, 0, densFuncBG, rimWidth, rimHeight, 0-rimDist, rimWidth, rimHeight, 0+rimDist);
        % residual of the fitted model function
        res=sum((sample_Y_convolved-histCounts).^2)/sum(histCounts.^2);      % mean squared error as the residue
        %res=sum((sample_Y_convolved-histCounts).^2)/length(modelFunc_sampling_X);
    end

    function res=fitDist_gap_GaussianConv(histCounts, multiPrecBool, locPrecision, ThompsonPrec_filtered, modelFunc_sampling_dX, linkerRad, linkerType, modelFunc_sampling_X_init, modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, modelFunc_dX, dX_refinement, histBinN, rimWidth, x )
        % function to fit the model function to histogram data of the measured distances
        
        % parameters to fit
        densFuncSize=x(1);
        densFuncMax=x(2);
        densFuncBG=x(3);
        if rimWidth==0
            rimHeight=0;
            GaussianConvSigma=x(4);
        else
            rimHeight=x(4);
            GaussianConvSigma=x(5);
        end
        
        % calculate the function with that the selected dye density function shall be convolved in order to calculate the model function, accounts for the aforementioned inaccuracies
        if multiPrecBool==false
            if isempty(locPrecision)
                locPrecision=mean(ThompsonPrec_filtered)+GaussianConvSigma;
            end
            [convFunc_gap, convRad_N_gap]=calcConvFunction(modelFunc_sampling_dX, linkerRad, locPrecision+GaussianConvSigma, linkerType);
        else
            [convFunc_gap, convRad_N_gap]=calcConvFunction(modelFunc_sampling_dX, linkerRad, ThompsonPrec_filtered+GaussianConvSigma, linkerType);
        end
        
        % extending the "x" coordinates because the convolution
        modelFunc_sampling_X_extend_temp_gap=(1:convRad_N_gap)'*modelFunc_sampling_dX;
        modelFunc_sampling_X_gap=[modelFunc_sampling_X_init(1)-modelFunc_sampling_X_extend_temp_gap; modelFunc_sampling_X_init ;modelFunc_sampling_X_init(end)+modelFunc_sampling_X_extend_temp_gap];
        
%         densFuncSize=110;
%         densFuncMax=-60;
%         densFuncBG=92;
%         rimHeight=x(4);
        
        rimDist=(densFuncSize+rimWidth)/2;
        
        % values of the model function
        sample_Y_convolved=calcModelFuncConvolved( convFunc_gap, convRad_N_gap, 'Rectangular gap', modelFunc_sampling_X_gap, modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, modelFunc_dX, dX_refinement, histBinN, densFuncSize, densFuncMax, 0, 0, densFuncBG, rimWidth, rimHeight, 0-rimDist, rimWidth, rimHeight, 0+rimDist);
        % residual of the fitted model function
        res=sum((sample_Y_convolved-histCounts).^2)/sum(histCounts.^2);      % mean squared error as the residue
        %res=sum((sample_Y_convolved-histCounts).^2)/length(modelFunc_sampling_X_gap);
    end

%% Reading the measured data of line diameter

% double array containing the distance data and the precision of the
% localizations
locLineDist=lineDiam_histData.locLineDist;
ThompsonPrec=lineDiam_histData.ThompsonPrec;

% cell array containing the histogram edges of the different lines
histEdges_cell=lineDiam_histData.histEdges;

% cell array containing the histogram counts of the different lines
histCounts=lineDiam_histData.histCounts;

%% Other input parameters
if length(varargin)==1
    
    dialogOutput=lineDiameter_fit_inputDialog( ThompsonPrec );
    
    modelFuncType=dialogOutput.modelFuncType;
    linkerRad=dialogOutput.linkerLength;
    precBounds=dialogOutput.precBounds;
    locPrecision=dialogOutput.singlePrecValue;
    multiPrecBool=dialogOutput.multiPrecBool;
    initFitVals=varargin{1};
elseif length(varargin)==7
    modelFuncType=varargin{1};
    linkerRad=varargin{2};
    precBounds=varargin{3};
    locPrecision=varargin{4};
    multiPrecBool=varargin{5};
    initFitVals=varargin{6};
else
    error('Number of the function inputs is not correct.')
end
iterGaussianConvBool=varargin{7};

%% Preparing the measured data of line diameter

% number of the different lines
histEdges_cellN=numel(histEdges_cell);

if precBounds(1)==-Inf && precBounds(2)==Inf
   
    ThompsonPrec_filtered=ThompsonPrec(~isnan(ThompsonPrec));
    locLineDist_filtered=locLineDist(~isnan(locLineDist));
    
    % determining whether the histogram edges matches for all the different lines
    matchingHist_bool=true;
    if histEdges_cellN>1
        for idxLine=2:histEdges_cellN
            if numel(histEdges_cell{1,1})==numel(histEdges_cell{idxLine,1})
                matchingHist_boolVect=histEdges_cell{1,1}==histEdges_cell{idxLine,1};
                matchingHist_bool=prod(matchingHist_boolVect);
            else
                matchingHist_bool=false;
            end
        end
        
        % if they do not match
        if ~matchingHist_bool
            error('There is a problem with the histogram data, histogram edges does not match for the different lines.')
        end
        
    end
    
    % if the histogram edges matches for all the different lines
    if matchingHist_bool
        histEdges=histEdges_cell{1};
        
        % calculate the mean values of the histogram counts for all the
        % different lines
        histCounts_cell=histCounts;
        histCounts_mat=cell2mat(histCounts_cell);
        histCounts=mean(histCounts_mat, 1);
    else
        histEdges=histEdges_cell{1};
        histCounts=histcounts_custom(locLineDist, histEdges);
    end
    
else
    locFilterBool=true(size(locLineDist));
    locFilterBool(isnan(locLineDist))=false;
    locFilterBool(ThompsonPrec<precBounds(1) | ThompsonPrec>precBounds(2))=false;
    
    ThompsonPrec_filtered=ThompsonPrec(locFilterBool);
    locLineDist_filtered=locLineDist(locFilterBool);
    
    histEdges=histEdges_cell{1};
    histCounts=histcounts_custom(locLineDist_filtered, histEdges);
    
end

histCounts=histCounts';
histEdges=histEdges';

% sampling values of the histogram data of the measured distances
%histEdges=linspace(-200,200,numel(histCounts))';
histEdgesN=numel(histEdges);
histBinN=numel(histEdges)-1;
hist_dX=(histEdges(end)-histEdges(1))/(histEdgesN-1);
hist_X=linspace(-(histBinN-1)*hist_dX/2,(histBinN-1)*hist_dX/2, histBinN)';

% initial sampling (central) points of the model function
dX_refinement=10;
modelFunc_sampling_N=histBinN*dX_refinement;
modelFunc_sampling_dX=hist_dX/dX_refinement;
modelFunc_sampling_X_init=linspace(-(modelFunc_sampling_N-1)*modelFunc_sampling_dX/2,(modelFunc_sampling_N-1)*modelFunc_sampling_dX/2, modelFunc_sampling_N)';

% left and right boundaies of the initial sampling regions (the regons belonging to each sampling points) of the model function
modelFunc_sampBoundaries_N=modelFunc_sampling_N+1;
modelFunc_sampBoundaries=linspace(-(modelFunc_sampBoundaries_N-1)*modelFunc_sampling_dX/2,(modelFunc_sampBoundaries_N-1)*modelFunc_sampling_dX/2, modelFunc_sampBoundaries_N)';
modelFunc_sampBoundaries_left=modelFunc_sampBoundaries(1:end-1);
modelFunc_sampBoundaries_right=modelFunc_sampBoundaries(2:end);

%% Fitting the model function
if ~iterGaussianConvBool    % if the iteration should also run on the width of the Gaussian convolution
    % calculate the function with that the selected dye density function shall be convolved in order to calculate the model function, accounts for the aforementioned inaccuracies
    if multiPrecBool==false
        if isempty(locPrecision) 
            locPrecision=mean(ThompsonPrec_filtered);
        end
        [convFunc, convRad_N]=calcConvFunction(modelFunc_sampling_dX, linkerRad, locPrecision, linkerType);
    else
        [convFunc, convRad_N]=calcConvFunction(modelFunc_sampling_dX, linkerRad, ThompsonPrec_filtered, linkerType);
    end
    % convFunc=ones(31,1);
    % convRad_N=15;
    
    % extending the "x" coordinates because the convolution
    modelFunc_sampling_X_extend_temp=(1:convRad_N)'*modelFunc_sampling_dX;
    modelFunc_sampling_X=[modelFunc_sampling_X_init(1)-modelFunc_sampling_X_extend_temp; modelFunc_sampling_X_init ;modelFunc_sampling_X_init(end)+modelFunc_sampling_X_extend_temp];
else
    % calculate the convolution function inside the function called by the
    % iteration and not here
end

switch sampleType
    case 'single line'

        % initial parameters of the selected dye density function for calculating the model function
        densFuncSize_init=2.0*nanstd(locLineDist_filtered);       % parameter determining its width
        densFuncBG_init=0;          % parameter for constant background
        densFuncMax_init=max(histCounts);   % parameter determining its peak value
        
        % fitting the model function to the histogram data of the measured
        % distances
        options = optimset('MaxIter',1000, 'TolFun', eps, 'TolX', eps);
        %options = optimset();
        [fittedParams,FVAL,EXITFLAG,OUTPUT]=fmincon(@(x) fitDist(histCounts, convFunc, convRad_N, modelFuncType, modelFunc_sampling_X, modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, modelFunc_sampling_dX, dX_refinement, histBinN, x), [densFuncSize_init; densFuncMax_init; densFuncBG_init], [], [], [], [], [0; 0; 0], [Inf; Inf; Inf], [], options);
        %[fittedParams,FVAL,EXITFLAG,OUTPUT]=fminsearch(@(x) fitDist(histCounts, convFunc, convRad_N, modelFuncType, modelFunc_sampling_X, modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, modelFunc_dX, dX_refinement, histBinN, x), [densFuncSize_init; densFuncMax_init; densFuncBG_init], options);
        
        % the fitted parameters of the selected dye density function
        densFuncSize_fitted=fittedParams(1);        % parameter determining its width, the FWHM
        densFuncMax_fitted=fittedParams(2);         % parameter determining its peak value
        densFuncBG_fitted=fittedParams(3);          % parameter for constant background
        
        % passing the fitted parameters into a table
        fittedParamsTable=tableReplacement.data(densFuncSize_fitted, densFuncMax_fitted, densFuncBG_fitted);
        
        %%
        locVar=var(locLineDist_filtered);
        
        if isempty(locPrecision) || locPrecision==0
            locPrecision=mean(ThompsonPrec_filtered);
        end
        
        switch linkerType
            case 'sphere'
                linker_std=linkerRad/sqrt(3);
                convFuncVar=locPrecision^2+linker_std^2;
            case 'Gaussian'
                linker_std=linkerRad/(2*sqrt(2*log(2)));
                convFuncVar=locPrecision^2+linker_std^2;
            otherwise
                error('Unknows linker type')
        end
        
        % calculating varince of the distances, correcting the value by
        % substracting the variance of the background, linker distribution and the
        % localization precision
        bgVar=((histEdges(end)-histEdges(1))/(2*sqrt(3)))^2;
        bgN=(length(histEdges)-1)*densFuncBG_fitted;
        locN=numel(locLineDist_filtered);
        locVar_bgVar_subs=(locVar*locN-bgVar*bgN)/(locN-bgN);   % substracting the variance of the background
        lineVar=locVar_bgVar_subs-convFuncVar;
        lineSTD=sqrt(lineVar);
        
        % calculating the line diameter from the variance of corrected variance of
        % the distances
        %mean(locLineDist_filtered)
        FWHM_from_var_Gauss=2*sqrt(2*log(2))*lineSTD;
        FWHM_from_var_disk=2*sqrt(3)*lineSTD;
        %FWHM_from_var_rectangular=2*sqrt(3)*lineSTD;

    case 'double lines'
        % fitting two guassian on the histogram for the initial values for
        % the iteration

        
        if exist('OCTAVE_VERSION', 'builtin') ~= 0
            % when run in Octave, detemine the initial parameters with and
            % external peak fitting tool
            % source: https://terpconnect.umd.edu/~toh/spectrum/InteractivePeakFitter.htm#command
            
            % fit two unconstrained Gaussians with linear baseline (could not find constant baseline setting...),
            % without plotting and with the default settings:
             [FitResults,FitError,baseline] = peakfit([hist_X, histCounts],0,0,2,0,0,0,0,1,0,0);
             if FitError(1)>5
                 % it th fit error is larger than 5%, try it without
                 % background removal
                 % I think the linear background removal does not work the
                 % way I think it should work...
                 % and the "peakfit" function does not run the same in
                 % Octave and in Matlab, it uses "fminseach" which I guess
                 % runs a slightly different step sizes
                 [FitResults2,FitError2,baseline2] = peakfit([hist_X, histCounts],0,0,2,0,0,0,0,0,0,0);
                 if FitError2(1)<FitError(1)
                     FitResults=FitResults2;
                     FitError=FitError2;
                     baseline=baseline2;
                 end
             end
             
             
             
             % flat baseline this should be used, but does not work:
            % [FitResults,~,baseline] = peakfit([hist_X, histCounts],0,0,2,0,0,0,0,3,0,0);
            
            % The first dimension of the "FittedResults" corresponds to the
            % fitted peak, the second dimension correspond to the fitted
            % parameters of the peaks
            % The order of the paramerers (along the second dimension):
            % FittedResults=['peak index', 'position', 'height', 'width', 'area']
            
            % the constant to convert the FWHM to the "standard deviation"
            % 2*sqrt(2*log(2))
            widthConst=1/2.3548;
            
            densFuncMax_init_1=FitResults(1,3);   % parameter determining its peak value
            densFuncMax_init_2=FitResults(2,3);   % parameter determining its peak value
            densFuncBG_init=0;          % parameter for constant background
            densFuncSize_init=(FitResults(1,4)*widthConst+FitResults(2,4)*widthConst)/2;       % parameter determining its width
            pos1_init=FitResults(1,2);       % initial position of the first line
            pos2_init=FitResults(2,2);       % initial position of the second line
        else
            % detemine the initial parameters with Matlab's built-in double
            % Gaussian fitting function
            
            f=fit(hist_X, histCounts,'gauss2');
            C=coeffvalues(f);
            
            % initial parameters of the selected dye density function for caclculating the model function
            %densFuncMax_init=(C(1,1)+C(1,4))/2;   % parameter determining its peak value
            densFuncMax_init_1=C(1,1);   % parameter determining its peak value
            densFuncMax_init_2=C(1,4);   % parameter determining its peak value
            densFuncBG_init=0;          % parameter for constant background
            densFuncSize_init=(sqrt(C(1,3)/2)+sqrt(C(1,6)/2))/2;       % parameter determining its width
            pos1_init=C(1,2);       % initial position of the first line
            pos2_init=C(1,5);       % initial position of the second line
        end
        
        % fitting the model function to the histogram data of the measured
        % distances
        options = optimset('MaxIter',1000, 'TolFun', eps, 'TolX', eps);
        %options = optimset();
        %[fittedParams,FVAL,EXITFLAG,OUTPUT]=fmincon(@(x) fitDist_twoLines(histCounts, convFunc, convRad_N, modelFuncType, modelFunc_sampling_X, modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, modelFunc_sampling_dX, dX_refinement, histBinN, x), [densFuncSize_init; densFuncMax_init; densFuncBG_init; pos1_init; pos2_init], [], [], [], [], [0; 0; 0; -Inf; -Inf], [Inf; Inf; Inf; Inf; Inf], [], options);
        [fittedParams,FVAL,EXITFLAG,OUTPUT]=fmincon(@(x) fitDist_twoLines_2(histCounts, convFunc, convRad_N, modelFuncType, modelFunc_sampling_X, modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, modelFunc_sampling_dX, dX_refinement, histBinN, x), [densFuncSize_init; densFuncMax_init_1; densFuncMax_init_2; densFuncBG_init; pos1_init; pos2_init], [], [], [], [], [0; 0; 0; 0; -Inf; -Inf], [Inf; Inf; Inf; Inf; Inf; Inf], [], options);
        %[fittedParams,FVAL,EXITFLAG,OUTPUT]=fminsearch(@(x) fitDist(histCounts, convFunc, convRad_N, modelFuncType, modelFunc_sampling_X, modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, modelFunc_dX, dX_refinement, histBinN, x), [densFuncSize_init; densFuncMax_init; densFuncBG_init], options);
        
        % the fitted parameters of the selected dye density function
%         densFuncSize_fitted=fittedParams(1);        % parameter determining its width, the FWHM
%         densFuncMax_fitted=fittedParams(2);         % parameter determining its peak value
%         densFuncBG_fitted=fittedParams(3);          % parameter for constant background
%         pos_1_fitted=fittedParams(4);
%         pos_2_fitted=fittedParams(5);
        densFuncSize_fitted=fittedParams(1);        % parameter determining its width, the FWHM
        densFuncMax_fitted_1=fittedParams(2);         % parameter determining its peak value
        densFuncMax_fitted_2=fittedParams(3);         % parameter determining its peak value
        densFuncBG_fitted=fittedParams(4);          % parameter for constant background
        pos_1_fitted=fittedParams(5);
        pos_2_fitted=fittedParams(6);
        lineDistance=abs(pos_1_fitted-pos_2_fitted);
        
        % passing the fitted parameters into a table
        %fittedParamsTable=table(densFuncSize_fitted, densFuncMax_fitted, densFuncBG_fitted, pos_1_fitted, pos_2_fitted, lineDistance);
         fittedParamsTable=tableReplacement.data(densFuncSize_fitted, densFuncMax_fitted_1, densFuncMax_fitted_2, densFuncBG_fitted, pos_1_fitted, pos_2_fitted, lineDistance);
        
    case 'gap'
        
        % initial parameters of the selected dye density function for caclculating the model function
        densFuncSides_init=(histCounts(1)+histCounts(end))/2;          % parameter for constant background
        centerIdx=floor(length(histCounts)/2);
        densFuncMiddle_init=histCounts(centerIdx);
        %if densFuncMiddle_init>densFuncSides_init
        %    densFuncMiddle_init=min(histCounts);
        %    disp('Could not give "densFuncMax_init" a proper value.')
        %end
        densFuncSize_init=initFitVals.gap.densFuncSize_init;       % parameter determining its width
        rimWidth=initFitVals.gap.rimWidth;
        
        if rimWidth==0
            rimHeight_init=0;
            
            % fitting the model function to the histogram data of the measured
            % distances
            options = optimset('MaxIter',1000, 'TolFun', eps, 'TolX', eps);
            %options = optimset();
            if ~iterGaussianConvBool
                [fittedParams,FVAL,EXITFLAG,OUTPUT]=fmincon(@(x) fitDist_gap(histCounts, convFunc, convRad_N, modelFunc_sampling_X, modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, modelFunc_sampling_dX, dX_refinement, histBinN, rimWidth, x), [densFuncSize_init; densFuncMiddle_init; densFuncSides_init], [], [], [], [], [0; 0; 0], [Inf; Inf; Inf], [], options);
                %[fittedParams,FVAL,EXITFLAG,OUTPUT]=fminsearch(@(x) fitDist(histCounts, convFunc, convRad_N, modelFuncType, modelFunc_sampling_X, modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, modelFunc_dX, dX_refinement, histBinN, x), [densFuncSize_init; densFuncMax_init; densFuncBG_init], options);
                fittedParams(4)=0;
            else
                GaussianConvSigma_init=0;
                [fittedParams,FVAL,EXITFLAG,OUTPUT]=fmincon(@(x) fitDist_gap_GaussianConv(histCounts, multiPrecBool, locPrecision, ThompsonPrec_filtered, modelFunc_sampling_dX, linkerRad, linkerType, modelFunc_sampling_X_init, modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, modelFunc_sampling_dX, dX_refinement, histBinN, rimWidth, x), [densFuncSize_init; densFuncMiddle_init; densFuncSides_init; GaussianConvSigma_init], [], [], [], [], [0; 0; 0; 0], [Inf; Inf; Inf; Inf], [], options);
                fittedParams(5)=fittedParams(4);
                fittedParams(4)=0;
            end
        else
            rimAreaConst=50;
            rimHeight_init=abs(densFuncSides_init-densFuncMiddle_init)*rimAreaConst/rimWidth;
            
            % fitting the model function to the histogram data of the measured
            % distances
            options = optimset('MaxIter',1000, 'TolFun', eps, 'TolX', eps);
            %options = optimset();
            if ~iterGaussianConvBool
                [fittedParams,FVAL,EXITFLAG,OUTPUT]=fmincon(@(x) fitDist_gap(histCounts, convFunc, convRad_N, modelFunc_sampling_X, modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, modelFunc_sampling_dX, dX_refinement, histBinN, rimWidth, x), [densFuncSize_init; densFuncMiddle_init; densFuncSides_init; rimHeight_init], [], [], [], [], [0; 0; 0; 0], [Inf; Inf; Inf; Inf], [], options);
                %[fittedParams,FVAL,EXITFLAG,OUTPUT]=fminsearch(@(x) fitDist(histCounts, convFunc, convRad_N, modelFuncType, modelFunc_sampling_X, modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, modelFunc_dX, dX_refinement, histBinN, x), [densFuncSize_init; densFuncMax_init; densFuncBG_init], options);
            else
                GaussianConvSigma_init=0;
                [fittedParams,FVAL,EXITFLAG,OUTPUT]=fmincon(@(x) fitDist_gap_GaussianConv(histCounts, multiPrecBool, locPrecision, ThompsonPrec_filtered, modelFunc_sampling_dX, linkerRad, linkerType, modelFunc_sampling_X_init, modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, modelFunc_sampling_dX, dX_refinement, histBinN, rimWidth, x), [densFuncSize_init; densFuncMiddle_init; densFuncSides_init; rimHeight_init; GaussianConvSigma_init], [], [], [], [], [0; 0; 0; 0; 0], [Inf; Inf; Inf; Inf; Inf], [], options);
            end
        end
        
        
%         % the fitted parameters of the selected dye density function
        densFuncSize_fitted=fittedParams(1);        % parameter determining its width, the FWHM
        densFuncMax_fitted=fittedParams(2);         % parameter determining its peak value
        densFuncBG_fitted=fittedParams(3);          % parameter for constant background
        rimHeight_fitted=fittedParams(4);
        rimDist_fitted=(densFuncSize_fitted+rimWidth)/2;
        if iterGaussianConvBool
            GaussianConvSigma_fitted=fittedParams(5);
        end
        
        % passing the fitted parameters into a table
        if iterGaussianConvBool
            fittedParamsTable=tableReplacement.data(densFuncSize_fitted, densFuncMax_fitted, densFuncBG_fitted, rimHeight_fitted, rimDist_fitted, GaussianConvSigma_fitted);
        else
            fittedParamsTable=tableReplacement.data(densFuncSize_fitted, densFuncMax_fitted, densFuncBG_fitted, rimHeight_fitted, rimDist_fitted);
        end
        
% densFuncSize_fitted=127;        % parameter determining its width, the FWHM
%         densFuncMax_fitted=1.2*-93;         % parameter determining its peak value
%         densFuncBG_fitted=1.2*138.4;          % parameter for constant background
%         rimHeight_fitted=1.2*2*317.6;
%         rimDist_fitted=68.8;
%         if iterGaussianConvBool
%             GaussianConvSigma_fitted=1.37;
%         end
end
%% Visualizing the result

switch sampleType
    case 'single line'
        % calculate the values of the fitted model function
        sample_Y_convolved_fitted=calcModelFuncConvolved( convFunc, convRad_N, modelFuncType, modelFunc_sampling_X, modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, modelFunc_sampling_dX, dX_refinement, histBinN, densFuncSize_fitted, densFuncMax_fitted, densFuncBG_fitted, 0 );
        
        sample_Y_fitted=calcModelFunc( 0, modelFuncType, modelFunc_sampling_X_init, modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, densFuncSize_fitted, densFuncMax_fitted, densFuncBG_fitted, 0 );
    case 'double lines'
        % calculate the values of the fitted model function
%         sample_Y_convolved_fitted_1=calcModelFuncConvolved( convFunc, convRad_N, modelFuncType, modelFunc_sampling_X, modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, modelFunc_sampling_dX, dX_refinement, histBinN, densFuncSize_fitted, densFuncMax_fitted, densFuncBG_fitted, pos_1_fitted );
%         sample_Y_convolved_fitted_2=calcModelFuncConvolved( convFunc, convRad_N, modelFuncType, modelFunc_sampling_X, modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, modelFunc_sampling_dX, dX_refinement, histBinN, densFuncSize_fitted, densFuncMax_fitted, 0, pos_2_fitted );
        sample_Y_convolved_fitted_1=calcModelFuncConvolved( convFunc, convRad_N, modelFuncType, modelFunc_sampling_X, modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, modelFunc_sampling_dX, dX_refinement, histBinN, densFuncSize_fitted, densFuncMax_fitted_1, densFuncBG_fitted, pos_1_fitted );
        sample_Y_convolved_fitted_2=calcModelFuncConvolved( convFunc, convRad_N, modelFuncType, modelFunc_sampling_X, modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, modelFunc_sampling_dX, dX_refinement, histBinN, densFuncSize_fitted, densFuncMax_fitted_2, 0, pos_2_fitted );
        sample_Y_convolved_fitted=sample_Y_convolved_fitted_1+sample_Y_convolved_fitted_2;
        
%         sample_Y_fitted_1=calcModelFunc( 0, modelFuncType, modelFunc_sampling_X_init, modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, densFuncSize_fitted, densFuncMax_fitted, densFuncBG_fitted, pos_1_fitted );
%         sample_Y_fitted_2=calcModelFunc( 0, modelFuncType, modelFunc_sampling_X_init, modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, densFuncSize_fitted, densFuncMax_fitted, 0, pos_2_fitted );
        sample_Y_fitted_1=calcModelFunc( 0, modelFuncType, modelFunc_sampling_X_init, modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, densFuncSize_fitted, densFuncMax_fitted_1, densFuncBG_fitted, pos_1_fitted );
        sample_Y_fitted_2=calcModelFunc( 0, modelFuncType, modelFunc_sampling_X_init, modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, densFuncSize_fitted, densFuncMax_fitted_2, 0, pos_2_fitted );
        sample_Y_fitted=sample_Y_fitted_1+sample_Y_fitted_2;
    case 'gap'
        
        if iterGaussianConvBool
            % calculate the function with that the selected dye density function shall be convolved in order to calculate the model function, accounts for the aforementioned inaccuracies
            if multiPrecBool==false
                if isempty(locPrecision)
                    locPrecision=mean(ThompsonPrec_filtered)+GaussianConvSigma_fitted;
                end
                [convFunc, convRad_N]=calcConvFunction(modelFunc_sampling_dX, linkerRad, locPrecision+GaussianConvSigma_fitted, linkerType);
            else
                [convFunc, convRad_N]=calcConvFunction(modelFunc_sampling_dX, linkerRad, ThompsonPrec_filtered+GaussianConvSigma_fitted, linkerType);
            end
        end
        
        % extending the "x" coordinates because the convolution
        modelFunc_sampling_X_extend_temp=(1:convRad_N)'*modelFunc_sampling_dX;
        modelFunc_sampling_X=[modelFunc_sampling_X_init(1)-modelFunc_sampling_X_extend_temp; modelFunc_sampling_X_init ;modelFunc_sampling_X_init(end)+modelFunc_sampling_X_extend_temp];
        
        sample_Y_convolved_fitted=calcModelFuncConvolved( convFunc, convRad_N, 'Rectangular gap', modelFunc_sampling_X, modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, modelFunc_sampling_dX, dX_refinement, histBinN, densFuncSize_fitted, densFuncMax_fitted, 0, 0, densFuncBG_fitted, rimWidth, rimHeight_fitted, 0-rimDist_fitted, rimWidth, rimHeight_fitted, 0+rimDist_fitted);
        
        sample_Y_fitted=calcModelFunc( 0, 'Rectangular gap', modelFunc_sampling_X_init, modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, densFuncSize_fitted, densFuncMax_fitted, 0, 0, densFuncBG_fitted, rimWidth, rimHeight_fitted, 0-rimDist_fitted, rimWidth, rimHeight_fitted, 0+rimDist_fitted );
        
end

% plot the fitted model function and the histogram data of the measured
% distances
fittedHistFigure=figure('visible','off');
histogram_custom('BinEdges', histEdges', 'BinCounts', histCounts');
hold on;
% plot the fitted model function in red:
plot(modelFunc_sampling_X_init, sample_Y_fitted, 'LineWidth', 1.5, 'color', 'red')
% plot the fitted convolved function (that directly fits to the histogramd
% data) in orange like color (the one that is the second or so default in
% Matlab):
plot(hist_X, sample_Y_convolved_fitted, 'color', [0.9290, 0.6940, 0.1250], 'LineWidth', 1.5)

% figure()
% plot(hist_X, histCounts, hist_X, sample_Y_convolved_fitted, modelFunc_sampling_X_init, sample_Y_fitted, hist_X, histcounts_custom(randn(10000,1)*57, histEdges))

title(['Hist of distances, ', modelFuncType, ' fit, ', linkerType,' linker']);
xlabel('distance [nm]')
ylabel('N of localizations');

labelXPos=[0.58];
labelYPos=[0.91];
switch sampleType
    case 'single line'
        labelText={['FWHM fit: ', sprintf('%.4g', densFuncSize_fitted), 'nm'],...
            ['FWHM var: ', sprintf('%.4g/%.4g', FWHM_from_var_Gauss, FWHM_from_var_disk), 'nm'],...
            ['residual: ', sprintf('%.4g', FVAL)]};
    case 'double lines'
        labelText={['FWHM fit: ', sprintf('%.4g', densFuncSize_fitted), 'nm'],...
            ['line dist.: ', sprintf('%.4g', lineDistance), 'nm'],...
            ['residual: ', sprintf('%.4g', FVAL)]};
    case 'gap'
        if ~iterGaussianConvBool
            labelText={['gap width: ', sprintf('%.4g', densFuncSize_fitted), 'nm'],...
                ['residual: ', sprintf('%.4g', FVAL)]};
            
            ax = gca;
            %set(ax, 'XLim', [-Inf Inf]);
            set(ax, 'YLim', [0 1.5*max(max(sample_Y_convolved_fitted),max(histCounts))]);
        else
            labelText={['gap width: ', sprintf('%.4g', densFuncSize_fitted), 'nm'],...
                ['residual: ', sprintf('%.4g', FVAL)],...
                ['Gaussian blur: ', sprintf('%.4g', GaussianConvSigma_fitted), 'nm']};
            
            ax = gca;
            %set(ax, XLim, [-Inf Inf]);
            set(ax, YLim, [0 1.5*max(max(sample_Y_convolved_fitted),max(histCounts))]);
        end
end
text( labelXPos, labelYPos, labelText, 'Units', 'normalized')

% scatter(hist_X, histCounts)
% hold on
% scatter(hist_X, sample_Y_convolved_fitted)
% hold off


end

